@@ -25,9 +25,8 @@
 #' @importFrom stats ave frequency is.ts setNames
 #' @importFrom grDevices rgb
 #' @name diffobj-package
-#' @docType package
 
-NULL
+"_PACKAGE"
 
 # Because all these functions are so similar, we have constructed them with a
 # function factory.  This allows us to easily maintain consistent formals during
@@ -150,6 +149,9 @@ make_diff_fun <- function(capt_fun) {
 #' anything past the \code{color.mode} parameter.  Default values are specified
 #' as options so that users may configure diffs in a persistent manner.
 #' \code{\link{gdo}} is a shorthand function to access \code{diffobj} options.
+#' Examples of what you aspects of the display you can control include many of
+#' the basic adjustment offered by traditional diff programs, color schemes,
+#' panel headers/titles, and more.
 #'
 #' Parameter order after \code{color.mode} is not guaranteed.  Future versions
 #' of \code{diffobj} may add parameters and re-order existing parameters past
@@ -174,13 +176,10 @@ make_diff_fun <- function(capt_fun) {
 #' While \code{diffPrint} attempts to handle the default R behavior that wraps
 #' wide tables, the results are often sub-optimal.  A better approach is to set
 #' the \code{disp.width} parameter to a large enough value such that wrapping is
-#' not necessary, and a browser-based \code{pager}.  In the future we will add
-#' the capability to specify different capture widths and wrap widths so that
-#' this is an option for terminal output (see
-#' \href{https://github.com/brodieG/diffobj/issues/109}{issue 109}).
+#' not necessary, and a browser-based \code{pager}.  If this bothers  you see
+#' \href{https://github.com/brodieG/diffobj/issues/109}{issue 109}.
 #'
-#' One thing to keep in mind is that \code{diffPrint} is not designed to work
-#' with very large data frames.
+#' \code{diffPrint} is not designed to work with large data frames.
 #'
 #' @export
 #' @seealso \code{\link{diffObj}}, \code{\link{diffStr}},
@@ -225,7 +224,7 @@ make_diff_fun <- function(capt_fun) {
 #'       web page (see docs for \code{html.output} under \code{\link{Style}}).
 #'   }
 #'   Defaults to \dQuote{auto}.  See \code{palette.of.styles} for details
-#'   on customization, \code{\link{style}} for full control of output format.
+#'   on customization, \code{\link{Style}} for full control of output format.
 #'   See `pager` parameter for more discussion of Rstudio behavior.
 #' @param brightness character, one of \dQuote{light}, \dQuote{dark},
 #'   \dQuote{neutral}, useful for adjusting color scheme to light or dark
@@ -412,12 +411,12 @@ make_diff_fun <- function(capt_fun) {
 #'   HTML.
 #' @param term.colors integer(1L) how many ANSI colors are supported by the
 #'   terminal.  This variable is provided for when
-#'   \code{\link[=num_colors]{crayon::num_colors}} does not properly detect how
-#'   many ANSI colors are supported by your terminal. Defaults to return value
-#'   of \code{\link[=num_colors]{crayon::num_colors}} and should be 8 or 256 to
-#'   allow ANSI colors, or any other number to disallow them.  This only
-#'   impacts output format selection when \code{style} and \code{format} are
-#'   both set to \dQuote{auto}.
+#'   \code{\link[crayon:num_colors]{crayon::num_colors}} does not properly
+#'   detect how many ANSI colors are supported by your terminal. Defaults to
+#'   return value of \code{\link[crayon:num_colors]{crayon::num_colors}} and
+#'   should be 8 or 256 to allow ANSI colors, or any other number to disallow
+#'   them.  This only impacts output format selection when \code{style} and
+#'   \code{format} are both set to \dQuote{auto}.
 #' @param tar.banner character(1L), language, or NULL, used to generate the
 #'   text to display ahead of the diff section representing the target output.
 #'   If NULL will use the deparsed \code{target} expression, if language, will
@@ -460,8 +459,15 @@ make_diff_fun <- function(capt_fun) {
 #' @export
 #' @examples
 #' ## `pager="off"` for CRAN compliance; you may omit in normal use
-#' diffPrint(letters, letters[-5], pager="off")
-
+#' old.opt <- options(diffobj.pager='off'); on.exit(old.opt)
+#'
+#' diffPrint(letters, letters[-5])
+#'
+#' ## Change the default banner/title
+#' diffPrint(
+#'   letters, letters[-5],
+#'   tar.banner='COMPLETE', cur.banner='INCOMPLETE'
+#' )
 setGeneric(
   "diffPrint", function(target, current, ...) standardGeneric("diffPrint")
 )
